"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.replaceBase = replaceBase;
exports.replaceCanonical = replaceCanonical;
exports.replaceLinks = replaceLinks;
exports.replaceMeta = replaceMeta;
exports.substitute = substitute;
var _core = require("./core.js");
var _url = require("./url.js");
function replaceBase(doc, section) {
  var base;
  var head;
  var url = section.href;
  var absolute = url.indexOf("://") > -1;
  if (!doc) {
    return;
  }
  head = (0, _core.qs)(doc, "head");
  base = (0, _core.qs)(head, "base");
  if (!base) {
    base = doc.createElement("base");
    head.insertBefore(base, head.firstChild);
  }

  // Fix for Safari crashing if the url doesn't have an origin
  if (!absolute && typeof window !== "undefined" && window.location) {
    let parts = window.location.href.split("/");
    let directory = "";
    parts.pop();
    directory = parts.join("/");
    url = directory + url;
  }
  base.setAttribute("href", url);
}
function replaceCanonical(doc, section) {
  var head;
  var link;
  var url = section.canonical || section.href;
  if (!doc) {
    return;
  }
  head = (0, _core.qs)(doc, "head");
  link = (0, _core.qs)(head, "link[rel='canonical']");
  if (link) {
    link.setAttribute("href", url);
  } else {
    link = doc.createElement("link");
    link.setAttribute("rel", "canonical");
    link.setAttribute("href", url);
    head.appendChild(link);
  }
}
function replaceMeta(doc, section) {
  var head;
  var meta;
  var id = section.idref || section.href;
  if (!doc) {
    return;
  }
  head = (0, _core.qs)(doc, "head");
  meta = (0, _core.qs)(head, "link[property='dc.identifier']");
  if (meta) {
    meta.setAttribute("content", id);
  } else {
    meta = doc.createElement("meta");
    meta.setAttribute("name", "dc.identifier");
    meta.setAttribute("content", id);
    head.appendChild(meta);
  }
}

// TODO: move me to Contents
function replaceLinks(contents, fn) {
  let links = contents.querySelectorAll("a[href]");
  if (!links.length) {
    return;
  }
  let base = (0, _core.qs)(contents.ownerDocument, "base");
  let location = base ? base.getAttribute("href") : contents.ownerDocument.defaultView.location.href;
  let replaceLink = function (link) {
    var href = link.getAttribute("href");
    if (href.indexOf("mailto:") === 0) {
      return;
    }
    var absolute = href.indexOf("://") > -1;
    if ((0, _url.isAbsolute)(href)) {
      link.setAttribute("target", "_blank");
    } else {
      link.onclick = function () {
        let linkUrl = (0, _url.createUrl)(href, location);
        fn(linkUrl.toString());
        return false;
      };
    }
  }.bind(this);
  for (var i = 0; i < links.length; i++) {
    replaceLink(links[i]);
  }
}
function substitute(content, urls, replacements) {
  urls.forEach(function (url, i) {
    if (url && replacements[i]) {
      content = content.replace(new RegExp(url, "g"), replacements[i]);
    }
  });
  return content;
}