"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _core = require("./core.js");
var _url = require("./url.js");
function request(url, type, options = {}) {
  let settings = {
    method: 'GET',
    headers: options.headers,
    credentials: options.credentials
  };

  // If type isn"t set, determine it from the file extension
  if (!type) {
    type = (0, _url.extension)(url);
  }
  return fetch(url, settings).then(function (response) {
    let deferred = new _core.defer();
    if (response.ok) {
      return response;
    } else if (response.status === 403) {
      deferred.reject({
        status: this.response.status,
        response: this.response.response,
        message: "Forbidden",
        stack: new Error().stack
      });
      return deferred.promise;
    } else {
      deferred.reject({
        status: response.status,
        message: "Empty Response",
        stack: new Error().stack
      });
      return deferred.promise;
    }
  }).then(function (response) {
    if ((0, _core.isXml)(type)) {
      return response.text();
    } else if (type == "xhtml") {
      return response.text();
    } else if (type == "html" || type == "htm") {
      return response.text();
    } else if (type == "json") {
      return response.json();
    } else if (type == "blob") {
      return response.blob();
    } else if (type == "binary") {
      return response.arrayBuffer();
    } else {
      return response.text();
    }
  }).then(function (result) {
    let r;
    if ((0, _core.isXml)(type)) {
      r = (0, _core.parse)(result, "text/xml");
    } else if (type === "xhtml") {
      r = (0, _core.parse)(result, "application/xhtml+xml");
    } else if (type === "html" || type === "htm") {
      r = (0, _core.parse)(result, "text/html");
    } else if (type === "json") {
      r = result;
    } else if (type === "blob") {
      r = result;
    } else {
      r = result;
    }
    return r;
  });
}
var _default = request;
exports.default = _default;