"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createUrl = createUrl;
exports.directory = directory;
exports.extension = extension;
exports.filename = filename;
exports.isAbsolute = isAbsolute;
exports.relative = relative;
exports.resolve = resolve;
function isAbsolute(inputUrl) {
  if (!inputUrl) {
    return;
  }
  if (inputUrl instanceof URL) {
    return true;
  }
  return inputUrl.indexOf("://") > -1;
}
function createUrl(inputUrl, base) {
  if (inputUrl instanceof URL) {
    return inputUrl;
  } else if (!base && !isAbsolute(inputUrl)) {
    let locationBase = "";
    if (typeof window !== "undefined" && typeof window.location !== "undefined") {
      locationBase = window.location.href;
    }
    return new URL(inputUrl, locationBase);
  } else {
    return new URL(inputUrl, base);
  }
}
function filename(inputUrl) {
  const url = createUrl(inputUrl);
  return url.pathname.split('/').pop();
}
function directory(inputUrl) {
  const url = createUrl(inputUrl);
  const name = filename(url);
  return url.pathname.replace(name, "");
}
function extension(inputUrl) {
  const name = filename(inputUrl);
  return name.split('.').pop();
}
function resolve(inputUrl, path) {
  const url = createUrl(inputUrl);
  return new URL(path, url).href;
}
function relative(inputUrl, path) {
  const url = createUrl(inputUrl);
  return new URL(path, url).pathname;
}